package co.zsmb.rainbowcake.internal

import androidx.lifecycle.Lifecycle.Event.ON_START
import androidx.lifecycle.Lifecycle.Event.ON_STOP
import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.Observer
import co.zsmb.rainbowcake.internal.livedata.QueuedSingleShotLiveData
import co.zsmb.rainbowcake.util.LifecycleTest
import co.zsmb.rainbowcake.util.MockObserver
import org.junit.Before
import org.junit.Test

@Suppress("UsePropertyAccessSyntax")
class QueuedSingleShotLiveDataTest : LifecycleTest() {

    private val queuedLiveData: MutableLiveData<String> = QueuedSingleShotLiveData()

    private val observer = MockObserver<String>()

    @Before
    fun setUp() {
        queuedLiveData.observe(this, observer)
    }

    @Test
    fun startStopStart() {
        queuedLiveData.setValue("a")
        observer.assertObserved()

        lifecycle.handleLifecycleEvent(ON_START)
        observer.assertObserved("a")

        queuedLiveData.setValue("b")
        observer.assertObserved("b")

        queuedLiveData.setValue("c")
        lifecycle.handleLifecycleEvent(ON_STOP)
        queuedLiveData.setValue("d")
        queuedLiveData.setValue("e")
        observer.assertObserved("c")

        lifecycle.handleLifecycleEvent(ON_START)
        observer.assertObserved("d", "e")
    }

    @Test
    fun nullValue() {
        lifecycle.handleLifecycleEvent(ON_START)
        queuedLiveData.setValue("a")
        queuedLiveData.setValue(null)
        queuedLiveData.setValue("b")
        observer.assertObserved("a", null, "b")
    }

    @Test(expected = IllegalStateException::class)
    fun multipleObservers() {
        queuedLiveData.observe(this, Observer { })
    }

    @Test
    fun observeRemoveObserve() {
        lifecycle.handleLifecycleEvent(ON_START)
        queuedLiveData.setValue("a")
        observer.assertObserved("a")

        queuedLiveData.removeObserver(observer)
        queuedLiveData.setValue("c")
        queuedLiveData.setValue("d")
        observer.assertObserved()

        queuedLiveData.observe(this, observer)
        observer.assertObserved("c", "d")
    }

}
